#!/usr/bin/perl

#Version 1.00


use strict;

use HTTP::Proxy;
use HTTP::Proxy::BodyFilter::complete;

# initialisation
my $proxy = HTTP::Proxy->new( port => 3128 );
close STDERR;

# This logs the queries and clicks
$proxy->push_filter( request => LogClicks->new() );

# By forcing the client not to cache, we get all clicks, not just first-time clicks
# Depending on the setting, we might not want to do this.
$proxy->push_filter( response => NoCache->new() );

# Changes the user agent to avoid content from Bing being compressed
$proxy->push_filter( host => 'bing.com',  path => 'search', request => NoCompression->new() );

# This catches the results of Bing requests
$proxy->push_filter( host => 'bing.com',
					 response => HTTP::Proxy::BodyFilter::complete->new, 
					 response => LogQueries->new() );

$proxy->start;

############### CLICK LOGGING ################
{
    package LogClicks;
    use base qw( HTTP::Proxy::HeaderFilter );

    sub filter {
        my ( $self, $headers, $message ) = @_;

        my $url = $message->url;
		my $referer = $headers->header('Referer');
		
		# If the referer is Bing, its a follow on request
		# Only keep non-bing requests
		if ($referer =~ /bing.com\/search\?q=([^&]*)/ && $url !~ /bing.com/ && $url !~ /bing.net/) 
		{
			my $query = $1;
			my $date = scalar localtime;
			my $ip = $headers->header('X-Forwarded-For');
						
			print "CLICK\t$date\t$query\t$ip\t$url\n";
		}
    }
}

########### RESULT LOGGING ############
{
    package LogQueries;
    use base qw( HTTP::Proxy::BodyFilter );

    sub filter {
        my ( $self, $dataref, $response, $protocol, $buffer ) = @_;

        my $url = $response->base;
		
        # If its a Bing request, and its the last chunk of the request (all the data is present)
		# Sometimes this doesn't get called, and we miss a query due to HTTP weirdness.
        if ($url =~ /^http:\/\/www.bing.com\/search?/ && !defined($buffer)) 
		{
			# Log the query
			if ($url =~ /\?q=([^&]*)/) {
				my $query = $1;
				my $date = $response->header('Date');
				my $ip = $response->request->header('X-Forwarded-For');
								
				# Collect the results
				my $results = "";
				while ($$dataref =~ /<h3><a[^>]*href="(http[^"]*)"[^>]*SERP/g)
				{
					$results .= "$1;";
				}

				print "QUERY\t$date\t$query\t$ip\t$results\n";
			}			
        }		
    }
}

############## HEADER REWRITING ##############
{
    package NoCompression;
    use base qw( HTTP::Proxy::HeaderFilter );

    # changes the User-Agent header in all requests
    sub filter {
        my ( $self, $headers, $message ) = @_;

		$message->headers->header( User_Agent => 'Mozilla/4.0 (compatible) Perl proxy' );
		$message->headers->header( Accept => 'application/xml,text/xml' );
    }
}

{
	package NoCache;
    use base qw( HTTP::Proxy::HeaderFilter );

	# Tells the web browser not to cache pages where the referer is Bing
    sub filter {
        my ( $self, $headers, $message ) = @_;

		my $referer = $message->request->headers->header("referer");
		if ($referer =~ /bing/) {
			$message->headers->header( 'Cache-Control' => 'no-cache' );
		}
    }
}
